@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2002, 2003, 2006 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,6 +26,9 @@ struct _GpeTimeSel
   GtkWidget *label;
   GtkWidget *button;
   GtkWidget *popup;
+  GtkWidget *clock;
+
+  gboolean changing_time;
 };
 
 /**
